#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "proxytrace.h"
#include "endian.h"
 
#ifdef PC
#include <io.h>
#endif

void PrintEntry_Text(FILE *out_file, TEntry *entry, int noURL);
void PrintEntry_Squid(FILE *out_file, TEntry *entry, int swap);

enum { ifNone = 0, ifDECV1_0, ifDECV1_2 } InputFormat = ifNone;
enum { ofNone = 0, ofDECText, ofSquid, ofSquidSwapped } OutputFormat = ofNone;

int getInputFormat(char* argv[]) {

	if (strncmp("-i", argv[0], 3) != 0)
		return ifNone;

	if (strncmp("v1.0", argv[1], 5) == 0)
		InputFormat = ifDECV1_0;
	else
	if (strncmp("v1.2", argv[1], 5) == 0)
		InputFormat = ifDECV1_2;
	else
		InputFormat = ifNone;

	return InputFormat;
}

int getOutputFormat(char* argv[]) {

	if (strncmp("-o", argv[0], 3) != 0)
		return ofNone;

	if (strncmp("txt", argv[1], 4) == 0)
		OutputFormat = ofDECText;
	else
	if (strncmp("squid", argv[1], 6) == 0)
		OutputFormat = ofSquid;
	else
	if (strncmp("squidsw", argv[1], 8) == 0)
		OutputFormat = ofSquidSwapped;
	else
		OutputFormat = ofNone;

	return OutputFormat;
}


int main (int argc, char* argv[]) {

	int    is_pipe = 0;
	int    is_little_endian = 0;
	char   *in_file_name = 0;
	FILE   *in_file = 0;
	char   line[1024];
	
	TEntry entry;

	/* parse command line */
	if (argc != 6 || !getInputFormat(argv+1) || !getOutputFormat(argv+3))  {
		fprintf(stderr, "usage: %s -i <v1.0|v1.2> -o <txt|squid|squidsw> <input_file>\n", argv[0]);
		exit(-1);
	}
	in_file_name = argv[argc-1];

	/* open input file */
	is_pipe = strncmp (".gz", in_file_name+strlen(in_file_name)-3, 4) == 0;

	if (is_pipe) {
		sprintf(line, "gunzip -c %s", in_file_name);
		in_file_name = line;

#ifdef PC
			in_file = _popen(line, "r");
#else
			in_file = popen(line, "r");
#endif

	} else
	if (strncmp("-", in_file_name, 2) == 0) {
		in_file = stdin;
	} else {
		in_file = fopen(in_file_name,"r");
	}

	if( !in_file ) {
		fprintf(stderr, "%s:%d: cannot open input file '%s'\n", 
			__FILE__, __LINE__, in_file_name);
		exit (-1);
	}

	/* determine endian-ness */
	is_little_endian = IsLittleEndian();

	/* read trace header */
	ReadHeader(in_file, 0);

	/* read entries untill EOF */
	while ((InputFormat == ifDECV1_0) ? ReadEntryV1(in_file, &entry) : ReadEntry(in_file, &entry)) {

		if ( !is_little_endian )
			ToOtherEndian(&entry);

		if (OutputFormat == ofDECText) 
			PrintEntry_Text(stdout, &entry, InputFormat == ifDECV1_0);
		else 
			PrintEntry_Squid(stdout, &entry, OutputFormat == ofSquidSwapped);

		/* fprintf(stderr, "%s:%d: got here\n", __FILE__, __LINE__); */
	}

	/* close input file if needed */
	if (in_file != stdin)  {

		if ( !is_pipe ) {
			fclose (in_file);
		} else {

#ifdef PC
				_pclose(in_file);
#else 
				pclose(in_file);
#endif
		 }
	}

	return 0;
}
